@@ -24,7 +24,7 @@ class Check(SiteCommand):
             date = page.meta.get("date", None)
             if date is not None:
                 if not isinstance(date, datetime.datetime):
-                    log.info("%s: meta date %r is not an instance of datetime", page.src_relpath, date)
+                    log.info("%s: meta date %r is not an instance of datetime", page.src.relpath, date)
 
         for type, count in sorted(counts.items()):
             print("{} {} pages".format(count, type))
