@@ -1,4 +1,4 @@
-from staticsite.rendered import RenderedString
+from staticsite.render import RenderedString
 from staticsite import Page, Feature
 from staticsite.archetypes import Archetype
 import pytz
@@ -18,8 +18,24 @@ re_ext = re.compile(r"\.(json|toml|yaml)$")
 
 
 class DataPages(Feature):
-    def __init__(self, site):
-        super().__init__(site)
+    """
+    Handle datasets in content directories.
+
+    This allows to store pure-data datasets in JSON, Yaml, or Toml format in
+    the contents directory, access it from pages, and render it using Jinja2
+    templates.
+
+    Each dataset needs, at its toplevel, to be a dict with a ``type`` element,
+    and the dataset will be rendered using the ``data-{{type}}.html`` template.
+
+    Other front-matter attributes like ``date``, ``title``, ``aliases``,
+    ``series`` and taxonomy names are handled as with other pages. The rest of
+    the dictionary is ignored and can contain any data one wants.
+    """
+    RUN_BEFORE = ["tags"]
+
+    def __init__(self, *args, **kw):
+        super().__init__(*args, **kw)
         self.by_type = defaultdict(list)
         self.j2_globals["data_pages"] = self.jinja2_data_pages
         self.page_class_by_type = {}
@@ -27,19 +43,19 @@ class DataPages(Feature):
     def register_page_class(self, type: str, cls):
         self.page_class_by_type[type] = cls
 
-    def try_load_page(self, root_abspath, relpath):
-        mo = re_ext.search(relpath)
+    def try_load_page(self, src):
+        mo = re_ext.search(src.relpath)
         if not mo:
             return None
-        data = load_data(os.path.join(root_abspath, relpath), relpath, mo.group(1))
+        data = load_data(src, mo.group(1))
         if data is None:
             return None
         type = data.get("type", None)
         if type is None:
-            log.error("%s: data type not found: ignoring page", relpath)
+            log.error("%s: data type not found: ignoring page", src.relpath)
             return None
         cls = self.page_class_by_type.get(type, DataPage)
-        page = cls(self.site, root_abspath, relpath, data)
+        page = cls(self.site, src, data)
         data_type = page.meta.get("type")
         self.by_type[data_type].append(page)
         return page
@@ -64,12 +80,12 @@ class DataPages(Feature):
         return page_filter.filter(self.by_type.get(type, []))
 
 
-def load_data(abspath, relpath, fmt):
-    with open(abspath, "rt") as fd:
+def load_data(src, fmt):
+    with open(src.abspath, "rt") as fd:
         try:
             return parse_data(fd, fmt)
         except Exception:
-            log.exception("%s: failed to parse %s content", relpath, fmt)
+            log.exception("%s: failed to parse %s content", src.relpath, fmt)
             return
 
 
@@ -122,28 +138,26 @@ class DataPage(Page):
     TYPE = "data"
     FINDABLE = True
 
-    def __init__(self, site, root_abspath, relpath, data):
-        dirname, basename = os.path.split(relpath)
+    def __init__(self, site, src, data):
+        dirname, basename = os.path.split(src.relpath)
         if basename.startswith("index.") or basename.startswith("README."):
             linkpath = dirname
         else:
-            linkpath = os.path.splitext(relpath)[0]
+            linkpath = os.path.splitext(src.relpath)[0]
         super().__init__(
             site=site,
-            root_abspath=root_abspath,
-            src_relpath=relpath,
+            src=src,
             src_linkpath=linkpath,
             dst_relpath=os.path.join(linkpath, "index.html"),
             dst_link=os.path.join(site.settings.SITE_ROOT, linkpath))
 
         # Read and parse the contents
-        src = self.src_abspath
-        if src is None:
+        if self.src.stat is None:
             if self.meta.get("date", None) is None:
                 self.meta["date"] = self.site.generation_time
         else:
             if self.meta.get("date", None) is None:
-                self.meta["date"] = pytz.utc.localize(datetime.datetime.utcfromtimestamp(os.path.getmtime(src)))
+                self.meta["date"] = pytz.utc.localize(datetime.datetime.utcfromtimestamp(self.src.stat.st_mtime))
 
         self.meta.update(data)
         self.data = data
@@ -167,7 +181,7 @@ class DataPage(Page):
                 except jinja2.TemplateNotFound:
                     pass
                 except Exception:
-                    log.exception("%s: cannot load template %s", self.src_relpath, template_name)
+                    log.exception("%s: cannot load template %s", self.src.relpath, template_name)
                     return "cannot load template {}".format(template_name)
 
             if template is None:
@@ -175,7 +189,7 @@ class DataPage(Page):
                 try:
                     template = self.site.theme.jinja2.get_template("data.html")
                 except Exception:
-                    log.exception("%s: cannot load template", self.src_relpath)
+                    log.exception("%s: cannot load template", self.src.relpath)
                     return "cannot load template data.html"
 
             self._content = template.render(
@@ -239,3 +253,8 @@ class DataArchetype(Archetype):
             post_body = fd.getvalue()
 
         return archetype_meta, post_body
+
+
+FEATURES = {
+    "data": DataPages,
+}
