@@ -1,4 +1,8 @@
 import os
+
+# Root directory used to resolve relative path in site sources
+PROJECT_ROOT = "."
+
 # Root directory of the site in the URLs we generate.
 #
 # If you are publishing the site at /prefix instead of root of the domain,
@@ -39,4 +43,20 @@ MARKDOWN_EXTENSIONS = [
     "markdown.extensions.codehilite",
     "markdown.extensions.fenced_code",
 ]
-MARKDOWN_EXTENSION_CONFIGS = {}
+MARKDOWN_EXTENSION_CONFIGS = {
+    'markdown.extensions.extra': {
+        'markdown.extensions.footnotes': {
+            # See https://github.com/spanezz/staticsite/issues/13
+            'UNIQUE_IDS': True,
+        },
+    },
+}
+
+# List of asset directories included from /usr/share/javascript
+SYSTEM_ASSETS = []
+
+# If true, do not ignore pages with dates in the future
+DRAFT_MODE = False
+
+# If True, store cached data to speed up rebuilds
+CACHE_REBUILDS = True
