@@ -27,7 +27,7 @@ class PageFilter:
                 self.sort_reverse = False
 
         self.taxonomy_filters = []
-        for taxonomy in self.site.features["taxonomies"].taxonomies:
+        for taxonomy in self.site.features["tags"].taxonomies:
             t_filter = kw.get(taxonomy.name)
             if t_filter is None:
                 continue
@@ -41,7 +41,7 @@ class PageFilter:
         for page in all_pages:
             if not page.FINDABLE:
                 continue
-            if self.re_path is not None and not self.re_path.match(page.src_relpath):
+            if self.re_path is not None and not self.re_path.match(page.src.relpath):
                 continue
             if self.sort is not None and self.sort != "url" and self.sort not in page.meta:
                 continue
@@ -79,6 +79,9 @@ class Theme:
         # Load feature plugins from the theme directory
         self.load_features()
 
+        # We are done adding features
+        self.site.features.commit()
+
         # Jinja2 template engine
         from jinja2 import Environment, FileSystemLoader
         self.jinja2 = Environment(
@@ -109,7 +112,7 @@ class Theme:
         self.jinja2.filters["basename"] = self.jinja2_basename
 
         # Add feature-provided globals and filters
-        for feature in self.site.features.values():
+        for feature in self.site.features.ordered():
             self.jinja2.globals.update(feature.j2_globals)
             self.jinja2.filters.update(feature.j2_filters)
 
@@ -119,28 +122,10 @@ class Theme:
         """
         Load feature modules from the features/ theme directory
         """
-        import pkgutil
-        import importlib
         features_dir = self.root / "features"
         if not features_dir.is_dir():
             return
-
-        for module_finder, name, ispkg in pkgutil.iter_modules([features_dir.as_posix()]):
-            try:
-                spec = module_finder.find_spec(name)
-                mod = importlib.util.module_from_spec(spec)
-                spec.loader.exec_module(mod)
-            except Exception:
-                log.exception("%r: failed to load feature module", name)
-                continue
-
-            features = getattr(mod, "FEATURES", None)
-            if features is None:
-                log.warn("%r: feature module did not define a FEATURES dict", name)
-
-            # Register features with site
-            for name, cls in features.items():
-                self.site.add_feature(name, cls)
+        self.site.load_feature_dir([features_dir.as_posix()])
 
     def jinja2_basename(self, val):
         return os.path.basename(val)
@@ -166,7 +151,7 @@ class Theme:
             return dt.strftime(format)
         else:
             log.warn("%s+%s: invalid datetime format %r requested",
-                     context.parent["page"].src_relpath, context.name, format)
+                     context.parent["page"].src.relpath, context.name, format)
             return "(unknown datetime format {})".format(format)
 
     @jinja2.contextfunction
@@ -181,7 +166,7 @@ class Theme:
             cur_page = context.parent["page"]
             page = cur_page.resolve_link(arg)
             if page is None:
-                log.warn("%s+%s: unresolved link %s passed to url_for", cur_page.src_relpath, context.name, arg)
+                log.warn("%s+%s: unresolved link %s passed to url_for", cur_page.src.relpath, context.name, arg)
                 return ""
         else:
             page = arg
