@@ -1,15 +1,16 @@
-# coding: utf-8
 from unittest import TestCase
-from staticsite.build import Build
-from staticsite.site import Site
-from staticsite.core import Page
+from staticsite.cmd.build import Build
+from staticsite import Site
 from . import datafile_abspath, example_site, TestArgs, TestPage
 import os
 import datetime
 
+
 class TestSite(TestCase):
     def test_dirs(self):
         site = Site()
+        site.settings.THEME = datafile_abspath("theme")
+        site.load()
 
         page_root = TestPage(site, "page_root", date=datetime.datetime(2016, 1, 1))
         page_sub = TestPage(site, "dir1/page_sub", date=datetime.datetime(2016, 2, 1))
@@ -18,7 +19,6 @@ class TestSite(TestCase):
         site.add_page(page_root)
         site.add_page(page_sub)
         site.add_page(page_sub3)
-        site.load_theme(datafile_abspath("theme"))
         site.analyze()
 
         # We have a root dir index and dir indices for all subdirs
@@ -42,6 +42,7 @@ class TestSite(TestCase):
         self.assertEqual(dir_dir3.pages, [page_sub3])
         self.assertEqual(dir_dir3.subdirs, [])
 
+
 class TestMarkdownInJinja2(TestCase):
     def test_jinja2_markdown(self):
         with example_site() as root:
@@ -60,6 +61,7 @@ class TestMarkdownInJinja2(TestCase):
 
             self.assertEqual(content, '<p><em>This</em> is an <a href="http://example.org">example</a></p>')
 
+
 class TestBuild(TestCase):
     def test_dots(self):
         with example_site() as root:
